@@ -8,6 +8,7 @@ from typing import Union, List, Dict, Any, Optional, cast, Tuple
 from astrape.exceptions.exceptions import *
 from astrape.models.model_buildingblocks import Down, DoubleConv, Up
 import math
+import numpy as np
 
 
 # *****************************************************************  Classes  ***************************************************************** 
@@ -58,7 +59,9 @@ class _MLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        sample_weight : Optional[Union["np.array", "torch.Tensor"]] = None
+        activation_type : str = "relu", # "relu" or "leaky_relu"
+        negative_slope : Optional[float] = None, # negative slope for leaky relu   
+        sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None
     ) -> None:
 
         super(_MLP, self).__init__()
@@ -82,6 +85,8 @@ class _MLP(nn.Module):
         self.weight_decay = weight_decay 
         self.sample_weight = sample_weight
         self.l1_strength = l1_strength
+        self.activation_type = activation_type
+        self.negative_slope = negative_slope
         if self.n_layers == 1:
             if self.n_classes > 2:
                 layers.append(nn.Linear(self.dims, self.n_classes))
@@ -93,7 +98,17 @@ class _MLP(nn.Module):
             layers.append(fc1)
             if self.bn:
                 layers.append(nn.BatchNorm1d(num_features=self.n_hidden_units))
-            layers.append(nn.ReLU())
+            
+            if activation_type == "tanh":
+                layers.append(nn.Tanh())
+            elif activation_type == "relu":
+                layers.append(nn.ReLU())
+            elif activation_type == "leaky_relu":
+                if not negative_slope:
+                    negative_slope = 1e-2
+                layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+            else:
+                raise ValueError("Wrong name {} for activation function.".format(activation_type))
             layers.append(nn.Dropout(self.dropout_p))
             
             for i in range(self.n_layers-2):
@@ -101,7 +116,17 @@ class _MLP(nn.Module):
                 layers.append(fc2)
                 if self.bn:
                     layers.append(nn.BatchNorm1d(num_features=self.n_hidden_units))
-                layers.append(nn.ReLU())
+                
+                if activation_type == "tanh":
+                    layers.append(nn.Tanh())
+                elif activation_type == "relu":
+                    layers.append(nn.ReLU())
+                elif activation_type == "leaky_relu":
+                    if not negative_slope:
+                        negative_slope = 1e-2
+                    layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+                else:
+                    raise ValueError("Wrong name {} for activation function.".format(activation_type))
                 layers.append(nn.Dropout(self.dropout_p))
 
             
@@ -131,7 +156,7 @@ class MLP(BaseNet):
         self,
         dims : Union[int, Tuple[int]],
         n_classes : int,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         **hparams : Any
     ) -> None:
         
@@ -148,7 +173,9 @@ class MLP(BaseNet):
             'sample_weight',
             'lr',
             'weight_decay',
-            'l1_strength'
+            'l1_strength',
+            'activation_type',
+            'negative_slope'
             ]
         default_parameter_dict = {
             'n_layers' : 2,
@@ -243,7 +270,9 @@ class _ContractingMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None
+        activation_type : str = "relu",
+        negative_slope : Optional[float] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None:
         super(_ContractingMLP, self).__init__()
 
@@ -265,7 +294,8 @@ class _ContractingMLP(nn.Module):
         self.sample_weight = sample_weight
         self.l1_strength = l1_strength
         self.current_units = self.dims
-
+        self.activation_type = activation_type
+        self.negative_slope = negative_slope
         
         
         layers = []
@@ -282,7 +312,16 @@ class _ContractingMLP(nn.Module):
             layers.append(fc1)
             if self.bn:
                 layers.append(nn.BatchNorm1d(num_features=self.current_units))
-            layers.append(nn.ReLU())
+            if activation_type == "tanh":
+                layers.append(nn.Tanh())
+            elif activation_type == "relu":
+                layers.append(nn.ReLU())
+            elif activation_type == "leaky_relu":
+                if not negative_slope:
+                    negative_slope = 1e-2
+                layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+            else:
+                raise ValueError("Wrong name {} for activation function.".format(activation_type))
             
             layers.append(nn.Dropout(self.dropout_p))
 
@@ -295,7 +334,17 @@ class _ContractingMLP(nn.Module):
                 layers.append(fc2)
                 if self.bn:
                     layers.append(nn.BatchNorm1d(num_features=self.current_units))
-                layers.append(nn.ReLU())
+
+                if activation_type == "tanh":
+                    layers.append(nn.Tanh())
+                elif activation_type == "relu":
+                    layers.append(nn.ReLU())
+                elif activation_type == "leaky_relu":
+                    if not negative_slope:
+                        negative_slope = 1e-2
+                    layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+                else:
+                    raise ValueError("Wrong name {} for activation function.".format(activation_type))
                 
                 layers.append(nn.Dropout(self.dropout_p))
                 
@@ -324,7 +373,7 @@ class ContractingMLP(BaseNet):
         self,
         dims : Union[int, Tuple[int]],
         n_classes : int,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         **hparams : Any
     ) -> None:
         
@@ -339,7 +388,9 @@ class ContractingMLP(BaseNet):
             'bn',
             'lr',
             'weight_decay',
-            'l1_strength'
+            'l1_strength',
+            'activation_type',
+            'negative_slope'
             ]
         default_parameter_dict = {
             'contraction_ratio' : 0.5
@@ -433,8 +484,10 @@ class _CustomMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
+        activation_type : str = "relu",
+        negative_slope : Optional[float] = None,
         n_hidden_units_list : Optional[List[int]] = None,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None:
         super(_CustomMLP, self).__init__()
         if isinstance(dims, tuple):
@@ -453,6 +506,8 @@ class _CustomMLP(nn.Module):
         self.weight_decay = weight_decay  
         self.sample_weight = sample_weight
         self.l1_strength = l1_strength
+        self.activation_type = activation_type
+        self.negative_slope = negative_slope
 
         for n_hidden_units in self.n_hidden_units_list:
             if n_hidden_units <=0:
@@ -470,14 +525,32 @@ class _CustomMLP(nn.Module):
             layers.append(nn.Linear(self.dims, self.n_hidden_units_list[0]))
             if self.bn:
                 layers.append(nn.BatchNorm1d(num_features=self.n_hidden_units_list[0]))
-            layers.append(nn.ReLU())
+            if activation_type == "tanh":
+                layers.append(nn.Tanh())
+            elif activation_type == "relu":
+                layers.append(nn.ReLU())
+            elif activation_type == "leaky_relu":
+                if not negative_slope:
+                    negative_slope = 1e-2
+                layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+            else:
+                raise ValueError("Wrong name {} for activation function.".format(activation_type))
             layers.append(nn.Dropout(self.dropout_p))
             the_rest = self.n_hidden_units_list[1:]
             for idx in range(len(the_rest-1)):
                 layers.append(nn.Linear(the_rest[idx], the_rest[idx+1]))
                 if self.bn:
                     layers.append(nn.BatchNorm1d(num_features=self.the_rest[idx+1]))
-                layers.append(nn.ReLU())
+                if activation_type == "tanh":
+                    layers.append(nn.Tanh())
+                elif activation_type == "relu":
+                    layers.append(nn.ReLU())
+                elif activation_type == "leaky_relu":
+                    if not negative_slope:
+                        negative_slope = 1e-2
+                    layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+                else:
+                    raise ValueError("Wrong name {} for activation function.".format(activation_type))
                 layers.append(nn.Dropout(self.dropout_p))
             
             if self.n_classes > 2:
@@ -501,7 +574,7 @@ class CustomMLP(BaseNet):
         self,
         dims : Union[int, Tuple[int]],
         n_classes : int,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         **hparams : Any
     ) -> None:
         
@@ -571,7 +644,9 @@ class _VGG(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None
+        activation_type : str = "relu",
+        negative_slope : Optional[float] = None, 
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None: 
         
         super(_VGG, self).__init__()
@@ -599,13 +674,35 @@ class _VGG(nn.Module):
         self.weight_decay = weight_decay
         self.sample_weight = sample_weight
         self.l1_strength = l1_strength
+        self.activation_type = activation_type
+        self.negative_slope = negative_slope
     
         self.avgpool = nn.AdaptiveAvgPool2d((7, 7))
         layers = []
         layers.append(nn.Linear(512 * 7 * 7, 4096))
-        layers.append(nn.ReLU(inplace=False))
+        if activation_type == "tanh":
+            layers.append(nn.Tanh())
+        elif activation_type == "relu":
+            layers.append(nn.ReLU())
+        elif activation_type == "leaky_relu":
+            if not negative_slope:
+                negative_slope = 1e-2
+            layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+        else:
+            raise ValueError("Wrong name {} for activation function.".format(activation_type))
+
         layers.append(nn.Dropout(p=self.dropout_p))
-        layers.append(nn.ReLU(inplace=False))
+        layers.append(nn.Linear(4096, 4096))
+        if activation_type == "tanh":
+            layers.append(nn.Tanh())
+        elif activation_type == "relu":
+            layers.append(nn.ReLU())
+        elif activation_type == "leaky_relu":
+            if not negative_slope:
+                negative_slope = 1e-2
+            layers.append(nn.LeakyReLU(negative_slope=negative_slope))
+        else:
+            raise ValueError("Wrong name {} for activation function.".format(activation_type))
         layers.append(nn.Dropout(p=self.dropout_p))
         layers.append(nn.Linear(4096, n_classes))
         
@@ -635,7 +732,17 @@ class _VGG(nn.Module):
                 layers.append(nn.Conv2d(in_channels, v, kernel_size=3, padding=1))
                 if bn:
                     layers.append(nn.BatchNorm2d(v))
-                layers.append(nn.ReLU(inplace=False))
+                    
+                if self.activation_type == "tanh":
+                    layers.append(nn.Tanh())
+                elif self.activation_type == "relu":
+                    layers.append(nn.ReLU())
+                elif self.activation_type == "leaky_relu":
+                    if not negative_slope:
+                        negative_slope = 1e-2
+                    layers.append(nn.LeakyReLU(negative_slope=self.negative_slope))
+                else:
+                    raise ValueError("Wrong name {} for activation function.".format(self.activation_type))
                 in_channels = v
 
         return nn.Sequential(*layers)    
@@ -654,7 +761,7 @@ class VGG(ConvNet):
         self,
         dims : Union[int, Tuple[int]],
         n_classes : int,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         cfg : str ="VGG11",
         **hparams
     ) -> None:
@@ -671,7 +778,9 @@ class VGG(ConvNet):
             'lr',
             'weight_decay',
             'sample_weight',
-            'l1_strength'
+            'l1_strength',
+            'activation_type',
+            'negative_slope'
             ]
 
         default_parameter_dict = {}
@@ -744,8 +853,10 @@ class _UNet(nn.Module):
         features_start : int, 
         bilinear : bool,
         l1_strength : float,
+        activation_type : str = "relu",
+        negative_slope : Optional[float] = None,
         model_type : str = "UNet", # don't change this ,
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ):
         self.n_classes = n_classes
         self.batch_size = batch_size
@@ -759,6 +870,8 @@ class _UNet(nn.Module):
         self.bilinear = bilinear
         self.model_type = model_type
         self.sample_weight = sample_weight
+        self.activation_type = activation_type 
+        self.negative_slope = negative_slope
 
         if self.n_layers < 1:
             raise ValueError(f"n_layers = {self.n_layers}, expected: n_layers > 0")
@@ -780,7 +893,8 @@ class _UNet(nn.Module):
         self.input_channels = self.dims[0]
         super().__init__()
 
-        layers = [DoubleConv(self.input_channels, self.features_start, bn=self.bn)]
+        layers = [DoubleConv(self.input_channels, self.features_start, 
+                             bn=self.bn, activation_type=activation_type, negative_slope=negative_slope)]
 
         feats = self.features_start
         for _ in range(self.n_layers - 1):
@@ -812,7 +926,7 @@ class UNet(ConvNet):
         dims : Union[int, Tuple[int]],
         n_classes: int,
         model_type : str = "UNet", # don't change this
-        sample_weight : Optional[Union["torch.Tensor", "np.array"]] = None,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         **hparams : Any
     ):
 
@@ -829,7 +943,9 @@ class UNet(ConvNet):
             'lr' ,
             'weight_decay',
             'sample_weight',
-            'l1_strength'
+            'l1_strength',
+            'activation_type',
+            'negative_slope'
         ]
 
         default_parameter_dict = {
