@@ -39,13 +39,13 @@ def split_data(
         X_train, y_train = X, y
 
     if not isinstance(X_train, torch.Tensor):
-        X_predict_tensor = torch.tensor(X_train)
+        X_predict_tensor = torch.Tensor(X_train)
     if not isinstance(y_train, torch.Tensor):
-        y_predict_tensor = torch.tensor(y_train)
+        y_predict_tensor = torch.Tensor(y_train)
     if not isinstance(X_test, torch.Tensor):
-        X_test_tensor = torch.tensor(X_test)
+        X_test_tensor = torch.Tensor(X_test)
     if not isinstance(y_test, torch.Tensor):
-        y_test_tensor = torch.tensor(y_test)
+        y_test_tensor = torch.Tensor(y_test)
 
    
     X_train, X_val, y_train, y_val = train_test_split(
@@ -56,13 +56,13 @@ def split_data(
     )
     
     if not isinstance(X_train, torch.Tensor):
-        X_train_tensor = torch.tensor(X_train)
+        X_train_tensor = torch.Tensor(X_train)
     if not isinstance(y_train, torch.Tensor):
-        y_train_tensor = torch.tensor(y_train)
+        y_train_tensor = torch.Tensor(y_train)
     if not isinstance(X_val, torch.Tensor):
-        X_val_tensor = torch.tensor(X_val)
+        X_val_tensor = torch.Tensor(X_val)
     if not isinstance(y_val, torch.Tensor):
-        y_val_tensor = torch.tensor(y_val)
+        y_val_tensor = torch.Tensor(y_val)
 
     return X_train_tensor, y_train_tensor, X_val_tensor, y_val_tensor, X_predict_tensor, y_predict_tensor, X_test_tensor, y_test_tensor 
 
@@ -102,14 +102,14 @@ class DataModule(pl.LightningDataModule):
 
     def setup(self, stage):
         if self.X_val is not None and self.y_val is not None and self.X_test is not None and self.y_test is not None:
-            self.X_train_tensor = torch.tensor(self.X)
-            self.X_val_tensor = torch.tensor(self.X_val)
-            self.X_test_tensor = torch.tensor(self.X_test)
-            self.y_train_tensor = torch.tensor(self.y)
-            self.y_val_tensor = torch.tensor(self.y_val)
-            self.y_test_tensor = torch.tensor(self.y_test)
-            self.X_predict_tensor = torch.tensor(np.concatenate([self.X, self.X_val]))
-            self.y_predict_tensor = torch.tensor(np.concatenate([self.y, self.y_val]))
+            self.X_train_tensor = torch.Tensor(self.X)
+            self.X_val_tensor = torch.Tensor(self.X_val)
+            self.X_test_tensor = torch.Tensor(self.X_test)
+            self.y_train_tensor = torch.Tensor(self.y)
+            self.y_val_tensor = torch.Tensor(self.y_val)
+            self.y_test_tensor = torch.Tensor(self.y_test)
+            self.X_predict_tensor = torch.Tensor(np.concatenate([self.X, self.X_val]))
+            self.y_predict_tensor = torch.Tensor(np.concatenate([self.y, self.y_val]))
         else:
             self.X_train_tensor, self.y_train_tensor, self.X_val_tensor, self.y_val_tensor, \
                 self.X_predict_tensor, self.y_predict_tensor, self.X_test_tensor, self.y_test_tensor = \
