@@ -6,8 +6,8 @@ import torch.nn as nn
 import torch.nn.functional as F
 from torch import Tensor
 import matplotlib.pyplot as plt
+from torchmetrics import AUROC
 from torchmetrics.functional import dice_score
-
 from typing import Union, List, Dict, Any, Optional
 
 def column_or_1d(y):
@@ -17,7 +17,8 @@ def column_or_1d(y):
         return torch.ravel(y)
     
     raise ValueError("y should be a 1d tensor, got a tensor of shape {} instead.".format(shape))
-class BCELossAccuracy():
+
+class BCELossMetrics():
     def __init__(self):
         self.loss_function = nn.BCELoss()
     
@@ -27,7 +28,14 @@ class BCELossAccuracy():
             y_tilde = (y_hat > 0.5).int()
             accuracy = (y_tilde == labels.int()).float().mean().item()
         return accuracy
-    
+
+    @staticmethod
+    def auc(y_hat, labels):
+        with torch.no_grad():
+            auc_fn = AUROC()
+            auc = auc_fn(y_hat, labels)
+        return auc
+
     def __call__(
         self, 
         y_hat : "torch.Tensor", 
@@ -36,8 +44,10 @@ class BCELossAccuracy():
         
         loss = self.loss_function(y_hat, labels.reshape(-1).long()).squeeze()
         accuracy = self.accuracy(y_hat, labels)
-        return loss, accuracy
-class CELossAccuracy():
+        auc = self.auc(y_hat, labels)
+        return loss, accuracy, auc
+
+class CELossMetrics():
     def __init__(self):
         self.loss_function = nn.CrossEntropyLoss()
     
@@ -47,6 +57,13 @@ class CELossAccuracy():
             y_tilde = y_hat.argmax(axis=1)
             accuracy = (y_tilde == labels).float().mean().item()
         return accuracy
+
+    @staticmethod
+    def auc(y_hat, labels):
+        with torch.no_grad():
+            auc_fn = AUROC()
+            auc = auc_fn(y_hat, labels)
+        return auc
     
     def __call__(
         self, 
@@ -56,7 +73,8 @@ class CELossAccuracy():
         
         loss = self.loss_function(y_hat, labels.reshape(-1).long()).squeeze()
         accuracy = self.accuracy(y_hat, labels)
-        return loss, accuracy
+        auc = self.auc(y_hat, labels)
+        return loss, accuracy, auc
 
 class MSELoss():
     def __init__(self):
@@ -135,7 +153,7 @@ class R2Score():
         return loss
 
 
-class CEDiceLossAccuracy():
+class CEDiceLossMetrics():
     def __init__(self, n_classes):
         self.loss_function = nn.CrossEntropyLoss()
         self.n_classes = n_classes
@@ -146,6 +164,13 @@ class CEDiceLossAccuracy():
             accuracy = (y_tilde == labels).float().mean().item()
         return accuracy
 
+    @staticmethod
+    def auc(y_hat, labels):
+        with torch.no_grad():
+            auc_fn = AUROC()
+            auc = auc_fn(y_hat, labels)
+        return auc
+
     def __call__(
         self, 
         y_hat, 
@@ -163,7 +188,8 @@ class CEDiceLossAccuracy():
         """
         loss += dice_score(F.softmax(y_hat, dim=1).float(), labels.reshape(-1)).squeeze()
         accuracy = self.accuracy(y_hat, labels)
-        return loss, accuracy
+        auc = self.auc(y_hat, labels)
+        return loss, accuracy, auc
 
 
 
